﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// Robotti ja viholliset ovat molemmat kuolevaisia, mutta niiden kuollessa tapahtuu eri asioita.
// Kuoleminen kuitenkin tapahtuu molemmilla samalla tavalla, elämäpisteiden loputtua.
// Tämä skripti siis toimii pohjana näiden olioiden kuolemiselle.
// Merkitään tämä luokka abstraktiksi, sillä tämän skriptin on tarkoitus olla pohja olioiden
// ominaisille kuolemisskripteille. Abstrakti skripti ei toimi komponenttina, mutta siitä periytyvät skriptit toimivat.
public abstract class Kuoleminen : MonoBehaviour {

    // Merkitään elämäpisteet muuttuja yksityiseksi, koska emme halua muiden skriptien voivan muuttaa sen arvoa suoraan.
    // Oletusarvoksi asetetaan 100, eli elämäpisteiden ollessa 100 on oliolla täydet elämät.
    // SerializeField-attribuutti mahdollistaa yksityisen muuttujan näkemisen ja muokkaamisen objektin inspektorissa.
    [SerializeField]
    private float elämäpisteet = 100f;

    // Saadaksemme eri vahvuuksisia vihollisia, tarvitsemme vauriokerroin-muuttujan.
    // Oletuksena 1, muuta inspektorista objektikohtaisesti halutessasi. Mitä suurempi arvo, sitä heikompi olio on.
    public float vaurioKerroin = 1f;

    // Tämän julkisen Property-aksessorin kautta muutamme ylemmän yksityisen elämäpisteet-muuttujan arvoa. 
    public float Elämäpisteet
    {
        // Palautetaan get-aksessorissa elämäpisteet-muuttujan arvo.
        get
        {
            return elämäpisteet;
        }

        // Kun asetamme uuden arvon, set-aksessorin sisällä oleva koodi kutsutaan.
        // Syötetty uusi arvo on aksessorin sisällä toimivassa value-parametrissa.
        set
        {
            // Varastoidaan value-parametrin arvo, eli uusi elämäpisteiden arvo uusiArvo-muuttujaan.
            float uusiArvo = value;

            // Jos uusi elämäpisteiden arvo on pienempi kuin vanhempi, menettää olio elämäpisteitä ja on otettava vauriokerroin huomioon.
            if (uusiArvo < elämäpisteet)
            {
                // Lasketaan elämäpisteiden muutos vähentämällä uudesta arvosta vanha.
                float muutos = uusiArvo - elämäpisteet;

                // Asetetaan uusiArvo-muuttujalle vanha arvo, johon lisätään muutos kerrottuna vauriokertoimella.
                // Tällöin vauriokerroin on otettu huomioon elämäpisteiden vähennyksessä.
                uusiArvo = elämäpisteet + muutos * vaurioKerroin;
            }

            // Rajoitetaan elämäpisteiden arvo nollan ja sadan väliin Mathf-luokan Clamp-funktiolla.
            elämäpisteet = Mathf.Clamp(uusiArvo, 0f, 100f);

            // Jos elämäpisteet tippuivat nollaan, kutsu Kuole-funktio.
            if (elämäpisteet == 0f)
            {
                Kuole();
            }
        }
    }

    // Pelaajan ja vihollisen kuolemisen yhteydessä tapahtuu eri asioita, joten jätetään tämän funktion
    // täyttö niiden yksityiskohtaisempien kuolemisskriptien vastuulle.
    // Merkitään funktio suojatuksi, jolloin vain tämä ja tästä periytyvät skriptit voivat kutsua sen.
    // Abstraktiksi merkitty funktio jätetään tyhjäksi, mutta tästä skriptistä periytyvät skriptit joutuvat täyttämään sen.
    protected abstract void Kuole();

}
