﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class RobotinAmpuminen : MonoBehaviour {

    // Varastoidaan tykkien LineRenderer-komponentit tähän taulukkoon.
    // Merkitään se julkiseksi, jotta sen voi täyttää inspektorista.
    public LineRenderer[] tykit;

    // Määrittää laserin aiheuttaman vahingon määrän.
    public float ampumisLujuus = 50f;

    // Määrittää lasersäteen maksimikantaman.
    public float ampumisEtäisyys = 12f;

    // Määrittää tahdin, jolla robotti voi ampua.
    public float ampumisNopeus = 0.25f;

    // Määrittää lasersäteen lentonopeuden.
    public float laserSäteenNopeus = 50f;

    // Varastoidaan ajanhetki, jolloin robotti ampuu voidaksemme tahdittaa ampumista.
    float viimeAmpuminen;

    // Pidetään kirjaa monestiko robotti on ampunut, jolloin voimme määrittää kummalla
    // tykillä robotti seuraavaksi ampuu. 
    int ampumistenLukumäärä;

	void Update () {
        // Katsotaan, jos pelaaja pitää vasenta hiiren painiketta pohjassa.
        // Viime ampumisesta on täytynyt kulua vähintään ampumisNopeus-muuttujan verran aikaa.
        // Tämä lasketaan vähentämällä nykyisestä Time.time arvosta viimeAmpuminen-muuttujan arvo
        // ja verrataan sitä ampumisNopeus-muuttujan arvoon.
        if (Input.GetMouseButton(0) && Time.time - viimeAmpuminen >= ampumisNopeus)
        {
            // Ehtojen toteutuessa kutsutaan luomamme Ammu-funktio.
            Ammu();
        }

        // Lopuksi kutsutaan luomamme SäteidenLento-funktio, joka "siirtää" ammuttuja lasereita
        // eteenpäin maailmassa.
        SäteidenLento();
	}

    void Ammu()
    {
        // Valitaan aluksi seuraavaksi ammuttavan tykin LineRenderer-komponentti tykit-taulukosta.
        // Hyödynnetään ampumistenLukumäärä-muuttujan sisältämää tietoa.
        // Lisätään tähän muuttujaan ensin yksi ampumiskerta lisää. Tämä tapahtuu kirjoittamalla
        // int-tyyppisen muuttujamme eteen "++". Tällöin muuttujan arvoon lisätään yksi, ja uusi arvo
        // palautetaan. Tämä arvo kertoo monesko ampumiskerta tällähetkellä on kyseessä.
        // Saamme seuraavan tykin indeksin laskemalla jakojäännöksen, kun ampumiskerta jaetaan tykkien
        // lukumäärällä, eli kahdella. Käytetään tähän prosentti-merkkiä (%), joka tarkoittaa moduloa ohjelmoinnissa.
        // Tämä hakasulkeiden välissä oleva koodi palauttaa siis arvon 0 tai 1 vaihtelevasti, kun robotti ampuu.
        LineRenderer tykki = tykit[++ampumistenLukumäärä % 2];

        // Lähetämme Raycast-säteen ampumisen yhteydessä. Luodaan Ray säde, joka lähetetään tykistä eteenpäin.
        Ray säde = new Ray(tykki.transform.position, tykki.transform.forward);
        
        // Tarvitsemme säteen palauttamia tietoja, joten luodaan RaycastHit-tyyppinen muuttuja osuma.
        RaycastHit osuma;

        // Luodaan säteen päätepisteelle muuttuja, koska säde joko osuu johonkin objektiin tai
        // saavuttaa maksimietäisyydensä.
        Vector3 päätePiste;

        // Kutsutaan Raycast-funktio ehtolauseen sisällä, ja annetaan parametreiksi säde, out osuma ja
        // ampumisEtäisyys.
        if (Physics.Raycast(säde, out osuma, ampumisEtäisyys))
        {
            // Ehtolauseen toteutuessa, eli raycast-säde osuu objektiin, varastoidaan päätePiste-
            // muuttujaan osumakohta.
            päätePiste = osuma.point;

            // Katsotaan, jos objekti, johon säde osui, oli vihollinen. Kutsutaan objektin CompareTag-funktio,
            // joka vertaa objektille asetettua tagia parametrin arvoon. Olemme asettaneet vihollisobjekteille
            // "Vihollinen"-tagin niiden inspektorista.
            if (osuma.transform.CompareTag("Vihollinen"))
            {
                // Viholliseen osuessa hankitaan viittaus sen VihollisenKuoleminen-komponenttiin kutsumalla
                // sen GetComponent<>()-funktio. Vähennetään sen elämäpisteitä ampumisLujuus-muuttujan verran.
                osuma.transform.GetComponent<VihollisenKuoleminen>().Elämäpisteet -= ampumisLujuus;
            }
        }
        else // Jos raycast-säde ei osunut objektiin, se saavutti maksimietäisyydensä.
        {
            // Säteen saavuttaessa maksimietäisyydensä asetetaan päätePiste-muuttujan arvoksi
            // sijainti, johon säde ylsi. Tämä saadaan lisäämällä säteen lähtökohtaan säteen suunta
            // kerrottuna ampumisEtäisyys-muuttujalla.
            päätePiste = säde.origin + säde.direction * ampumisEtäisyys;
        }

        // Nyt tiedämme, mihin lasersäde yltää eli voimme laittaa sen näkyviin oikeaan kohtaan.
        // Kutsutaan LineRenderer tykki-muuttujan SetPositions-funktio, joka asettaa tässä tilanteessa
        // lasersäteen viivan alku- ja päätesijainnin. Syötetään parametriksi taulukkomuodossa esitetyt
        // sijaintivektorit säteen alkupaikka ja päätePiste-muuttujan arvo.
        tykki.SetPositions(new Vector3[] { säde.origin, päätePiste });

        // Lasersäde saadaan näkyviin asettamalla tykin, eli LineRenderer-komponentin, enabled arvoksi true.
        tykki.enabled = true;

        // Lopuksi asetetaan viimeAmpuminen-muuttujan arvoksi nykyinen Time.time arvo.
        viimeAmpuminen = Time.time;
    }

    void SäteidenLento()
    {
        // Iteroidaan tykit-taulukkoa foreach-silmukalla.
        foreach (LineRenderer tykki in tykit)
        {
            // Jos kyseessä oleva tykki, eli LineRenderer-komponentti ei ole päällä
            // skipataan tämä taulukon jäsen continue-komennolla.
            if (!tykki.enabled)
                continue;

            // Luodaan Vector3-tyyppinen taulukko pisteet, joka sisältää kaksi jäsentä.
            Vector3[] pisteet = new Vector3[2];

            // Täytetään pisteet-taulukko lasersäteen pisteillä kutsumalla LineRenderer-komponentin
            // GetPositions-funktio ja syötetään parametriksi luomamme taulukko.
            tykki.GetPositions(pisteet);

            // Taulukon nollannes jäsen on lasersäteen "hännän" sijainti. Siirretään häntää kohti säteen päätä
            // Vector3-luokan MoveTowards-funktiolla. Ensimmäinen parametri on piste, jota siirretään, eli säteen häntä.
            // Toinen parametri piste, jota kohti siirretään, eli säteen pää. Viimeinen parametri määrittää, kuinka paljon
            // häntä siirtyy kohti päätä.
            pisteet[0] = Vector3.MoveTowards(pisteet[0], pisteet[1], Time.deltaTime * laserSäteenNopeus);

            // Asetetaan viivan uudet sijainnit kutsumalla sen SetPositions-funktiota. 
            tykki.SetPositions(pisteet);

            // Lopuksi piilotamme viivan maailmasta, mikäli sen häntä on tarpeeksi lähellä sen päätä.
            // Verrataan viivan hännän ja pään sijaintien etäisyyttä esimerkiksi yhteen yksikköön.
            // Jos etäisyys on pienenmpi kuin yksi yksikkö, ehto toteutuu.
            // Ykkösen tilalla voi käyttää pienempää arvoa, mikä johtaa mahdollisesti hienompaan lopputulokseen.
            if (Vector3.Distance(pisteet[0], pisteet[1]) < 1f)
            {
                // Ehdon toteutuessa asetetaan viivan enabled arvoksi false.
                tykki.enabled = false;
            }
        }
    }
}
